package org.qatools.validator;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import org.qatools.info.Info;

//-testcase (happyPath) detail report stop recording details after first failure
//-support for parametized tst names

/**
 * <pre>
 * Scott Wilson presents
 * QATools.org
 * We Help You Master Your Test Automation
 * Specializing in Functional & Integration Test Automation using Maven Java Junit TestNG
 * 
 * 
 * Validator extends Junit by capturing and reporting all failures all at once.
 * This sppeds your QA effort by reducing the number of iterations.
 * 
 * Junit differs because it pukes at the first failed assertion.
 * Junit example - notice junit does not force useful error messages
 * 			assertTrue(false);		<=== FAILS IMMEDIATELY
 * 			assertEquals(1, 2)		
 * 			assertNotNull(null);	
 * 
 * Validator example - the API forces you to enter useful error messages
 * 			Validator v = new Validator();
 * 			v.assertTrue("Foo msg", false);
 * 			v.assertEquals("Bar msg", 1, 2);
 * 			v.assertNotNull("Apple", null);
 * 			v.junitAssertNoErrors();		<=== Reports everything at once!
 * 
 * </pre>
 * 
 * @author Scott Wilson - QATools.org
 */
public class Validator {
	private static Info info = new Info();
	
    // ------------------------------------------------------
    // ----- static initialization
    // ------------------------------------------------------

    static {
    	addPrintFormat(PrintFormat.BR);
    	addPrintFormat(PrintFormat.NEWLINE);
    	addPrintFormat(PrintFormat.NUMBER_ERRORS);    	
    	
    	setErrorOption(ErrorOption.SUREFIRE_FULL);
    	setDebugOption(DebugOption.EVERYWHERE);
    }
	
    // ------------------------------------------------------
    // ----- class variables
    // ------------------------------------------------------

    //errors
    private List<String> errors = null;
    
    //debug
    private List<String> debugs = null;
    
    // print formats
	private static List<PrintFormat> sPrintFormats;

    // error options
	private static ErrorOption sErrorOption;
	
    // debug options
	private static DebugOption sDebugOption;
	
    // ------------------------------------------------------
    // ----- constructors
    // ------------------------------------------------------

	public Validator() {
        //do nothing constructor
    }
    

    // ------------------------------------------------------
    // ----- Custom code
    // ------------------------------------------------------
    
	
    /**
     * Adds an error to the error list
     * 
     * @param error
     */
    public void addError(String error) {
    	if (errors == null) {
    		errors = new LinkedList<String>();
    	}
    	errors.add(error);
    }
	
    /**
     * Adds an info to the debug list
     * 
     * @param error
     */
    public void addDebug(String info) {
    	if (debugs == null) {
    		debugs = new LinkedList<String>();
    	}
    	debugs.add(info);
    }
    public void addDebug(String key, Object value) {
    	addDebug(key + " = " + value);
    }
    
    /**
     * Check if there are any errors
     * 
     * @return
     */
    public boolean doErrorsExist() {
    	if (errors==null || errors.size()==0) {
    		return false;
    	}
    	return true;
    }

    /**
     * Check if there are any debug messages
     * 
     * @return
     */
    public boolean doDebugsExist() {
    	if (debugs==null || debugs.size()==0) {
    		return false;
    	}
    	return true;
    }

    /**
     * If any errors where captured, throw an Exception
     */
    public void junitAssertNoErrors() throws Exception {
        StackTraceElement testCallerDetails = StackUnwinder.getCallerTestMethodDetails();
        StackTraceElement actualCallerDetails = StackUnwinder.getActualCallerDetails();

        String msg = null;
    	if (doErrorsExist()) {    		
    		StringBuffer sb = new StringBuffer();
			//sb.append("\n");			
			if (ErrorOption.SUREFIRE_SHORT.equals(sErrorOption)) {
				sb.append("\n<br>" + errors.get(0));
				sb.append("\n... more ... see QATools.org-detailReport.html");
			} else {
	    		formatErrors();
	    		for (String str : errors) {
	    			sb.append(str);
	    		}
			}

    		
    		if (doDebugsExist()) {
    			DetailReport.addDebugInfo(testCallerDetails, debugs);
    			if (DebugOption.EVERYWHERE.equals(sDebugOption)) {
            		formatDebugs();
                	for (String str : debugs) {
            			sb.append(str);
            		}
    			}
    		}
    		sb.append("\n");
    		msg = sb.toString();
    	}    	
		DetailReport.writeTestCaseReports(testCallerDetails);
		if (msg != null) {
    		throw new Exception(msg);
		}
    }
    
    /**
     * Returns the list of errors
     * 
     * @return
     */
    public List<String> getErrors() {
    	return errors;
    }
    
    /**
     * Returns the list of debug messages
     * 
     * @return
     */
    public List<String> getDebugs() {
    	return debugs;
    }
    
    /**
     * Resets there error & debug lists
     */
    public void reset() {
    	errors = null;
    	debugs = null;
    }
    
    
    // ------------------------------------------------------
    // ----- Re-implements these Junit APIs
    // ------------------------------------------------------

    
	/**
	 * Asserts that a condition is true. If it isn't it adds the error to the error list..
	 */
	public void assertTrue(String message, boolean condition) {
        StackTraceElement testCallerDetails = StackUnwinder.getCallerTestMethodDetails();
        StackTraceElement actualCallerDetails = StackUnwinder.getActualCallerDetails();

        if (!condition) {
        	DetailReport.recordFail(testCallerDetails, actualCallerDetails, message + " expected True but was False");
			addError(message + " expected True but was False");
		} else {
			DetailReport.recordPass(testCallerDetails, actualCallerDetails, message + " expected True and was True");
		}
	}
	

	
	/**
	 * Asserts that a condition is false. If it isn't it adds the error to the error list.
	 */
	public void assertFalse(String message, boolean condition) {
        StackTraceElement testCallerDetails = StackUnwinder.getCallerTestMethodDetails();
        StackTraceElement actualCallerDetails = StackUnwinder.getActualCallerDetails();

		if (condition) {
			addError(message + " expected False but was True");
        	DetailReport.recordFail(testCallerDetails, actualCallerDetails, message + " expected False but was True");
		} else {
			DetailReport.recordPass(testCallerDetails, actualCallerDetails, message + " expected False and was False");
		}
	}
	


	/**
	 * Adds the message to the error list
	 */
	public void fail(String message) {
        StackTraceElement testCallerDetails = StackUnwinder.getCallerTestMethodDetails();
        StackTraceElement actualCallerDetails = StackUnwinder.getActualCallerDetails();
    	DetailReport.recordFail(testCallerDetails, actualCallerDetails, message);
		addError(message);
	}



	/**
	 * Asserts that two objects are equal. If they are not
	 * add the message to the error list
	 */
	public void assertEquals(String message, Object expected, Object actual) {
        StackTraceElement testCallerDetails = StackUnwinder.getCallerTestMethodDetails();
        StackTraceElement actualCallerDetails = StackUnwinder.getActualCallerDetails();

		if (expected == null && actual == null) {
	    	DetailReport.recordPass(testCallerDetails, actualCallerDetails, message + " expected:'" + expected + "', actual:'" + actual + "'");
	    	return;
		}
		if (expected != null && expected.equals(actual)) {
	    	DetailReport.recordPass(testCallerDetails, actualCallerDetails, message + " expected:'" + expected + "', actual:'" + actual + "'");
			return;
		}
		addError(formatErrorMsg(message, expected, actual));
    	DetailReport.recordFail(testCallerDetails, actualCallerDetails, formatErrorMsg(message, expected, actual));
	}


	/**
	 * Asserts that two Strings are equal. 
	 */
	public void assertEquals(String message, String expected, String actual) {
        StackTraceElement testCallerDetails = StackUnwinder.getCallerTestMethodDetails();
        StackTraceElement actualCallerDetails = StackUnwinder.getActualCallerDetails();

		if (expected == null && actual == null) {
	    	DetailReport.recordPass(testCallerDetails, actualCallerDetails, message + " expected:'" + expected + "', actual:'" + actual + "'");
			return;
		}
		if (expected != null && expected.equals(actual)) {
	    	DetailReport.recordPass(testCallerDetails, actualCallerDetails, message + " expected:'" + expected + "', actual:'" + actual + "'");
			return;
		}
		addError(formatErrorMsg(message, expected, actual));
    	DetailReport.recordFail(testCallerDetails, actualCallerDetails, formatErrorMsg(message, expected, actual));
	}


	
	/**
	 * Asserts that two doubles are equal concerning a delta.  If the expected
	 * value is infinity then the delta value is ignored.
	 */
	public void assertEquals(String message, double expected, double actual, double delta) {
        StackTraceElement testCallerDetails = StackUnwinder.getCallerTestMethodDetails();
        StackTraceElement actualCallerDetails = StackUnwinder.getActualCallerDetails();

		// handle infinity specially since subtracting to infinite values gives NaN and the
		// the following test fails
		if (Double.isInfinite(expected)) {
			if (!(expected == actual)) {
				DetailReport.recordFail(testCallerDetails, actualCallerDetails, formatErrorMsg(message, new Double(expected), new Double(actual)));
				addError(formatErrorMsg(message, new Double(expected), new Double(actual)));
			}
		} else if (!(Math.abs(expected-actual) <= delta)) { // Because comparison with NaN always returns false
			DetailReport.recordFail(testCallerDetails, actualCallerDetails, formatErrorMsg(message, new Double(expected), new Double(actual)));
			addError(formatErrorMsg(message, new Double(expected), new Double(actual)));
		} else {
	    	DetailReport.recordPass(testCallerDetails, actualCallerDetails, message + " expected:'" + expected + "', actual:'" + actual + "'");
		}
	}
	


	/**
	 * Asserts that two floats are equal concerning a delta. If they are not
	 * an AssertionFailedError is thrown with the given message.  If the expected
	 * value is infinity then the delta value is ignored.
	 */
	public void assertEquals(String message, float expected, float actual, float delta) {
        StackTraceElement testCallerDetails = StackUnwinder.getCallerTestMethodDetails();
        StackTraceElement actualCallerDetails = StackUnwinder.getActualCallerDetails();

 		// handle infinity specially since subtracting to infinite values gives NaN and the
		// the following test fails
		if (Float.isInfinite(expected)) {
			if (!(expected == actual)) {
				addError(formatErrorMsg(message, new Float(expected), new Float(actual)));
				DetailReport.recordFail(testCallerDetails, actualCallerDetails, formatErrorMsg(message, new Float(expected), new Float(actual)));
			}
		} else if (!(Math.abs(expected-actual) <= delta)) {
			addError(formatErrorMsg(message, new Float(expected), new Float(actual)));			
			DetailReport.recordFail(testCallerDetails, actualCallerDetails, formatErrorMsg(message, new Float(expected), new Float(actual)));
		} else {
	    	DetailReport.recordPass(testCallerDetails, actualCallerDetails, message + " expected:'" + expected + "', actual:'" + actual + "'");
		}
	}

	
	/**
	 * Asserts that two longs are equal. If they are not
	 * an AssertionFailedError is thrown with the given message.
	 */
	public void assertEquals(String message, long expected, long actual) {
        StackTraceElement testCallerDetails = StackUnwinder.getCallerTestMethodDetails();
        StackTraceElement actualCallerDetails = StackUnwinder.getActualCallerDetails();

		if (expected != actual) {
	    	DetailReport.recordFail(testCallerDetails, actualCallerDetails, formatErrorMsg(message, new Long(expected), new Long(actual)));
	    	addError(formatErrorMsg(message, new Long(expected), new Long(actual)));
		} else
	    	DetailReport.recordPass(testCallerDetails, actualCallerDetails, message + " expected:'" + expected + "', actual:'" + actual + "'");
	}
	
	/**
	 * Asserts that two booleans are equal. If they are not
	 * an AssertionFailedError is thrown with the given message.
	 */
	public void assertEquals(String message, boolean expected, boolean actual) {
        StackTraceElement testCallerDetails = StackUnwinder.getCallerTestMethodDetails();
        StackTraceElement actualCallerDetails = StackUnwinder.getActualCallerDetails();

		if (expected != actual) {
	    	DetailReport.recordFail(testCallerDetails, actualCallerDetails, formatErrorMsg(message, new Boolean(expected), new Boolean(actual)));
	    	addError(formatErrorMsg(message, new Boolean(expected), new Boolean(actual)));
		} else {
	    	DetailReport.recordPass(testCallerDetails, actualCallerDetails, message + " expected:'" + expected + "', actual:'" + actual + "'");
		}
  	}
	
	/**
	 * Asserts that two bytes are equal. If they are not
	 * an AssertionFailedError is thrown with the given message.
	 */
  	public void assertEquals(String message, byte expected, byte actual) {
        StackTraceElement testCallerDetails = StackUnwinder.getCallerTestMethodDetails();
        StackTraceElement actualCallerDetails = StackUnwinder.getActualCallerDetails();

		if (expected != actual) {
	    	DetailReport.recordFail(testCallerDetails, actualCallerDetails, formatErrorMsg(message, new Byte(expected), new Byte(actual)));
	    	addError(formatErrorMsg(message, new Byte(expected), new Byte(actual)));
		} else {
	    	DetailReport.recordPass(testCallerDetails, actualCallerDetails, message + " expected:'" + expected + "', actual:'" + actual + "'");
		}
	}

  	
	/**
	 * Asserts that two chars are equal. If they are not
	 * an AssertionFailedError is thrown with the given message.
	 */
  	public void assertEquals(String message, char expected, char actual) {
        StackTraceElement testCallerDetails = StackUnwinder.getCallerTestMethodDetails();
        StackTraceElement actualCallerDetails = StackUnwinder.getActualCallerDetails();

		if (expected != actual) {
			addError(formatErrorMsg(message, new Character(expected), new Character(actual)));
	    	DetailReport.recordFail(testCallerDetails, actualCallerDetails, formatErrorMsg(message, new Character(expected), new Character(actual)));
		} else {
	    	DetailReport.recordPass(testCallerDetails, actualCallerDetails, message + " expected:'" + expected + "', actual:'" + actual + "'");
		}
  	}

	/**
	 * Asserts that two shorts are equal. If they are not
	 * an AssertionFailedError is thrown with the given message.
	 */
	public void assertEquals(String message, short expected, short actual) {
        StackTraceElement testCallerDetails = StackUnwinder.getCallerTestMethodDetails();
        StackTraceElement actualCallerDetails = StackUnwinder.getActualCallerDetails();

		if (expected != actual) {
			addError(formatErrorMsg(message, new Short(expected), new Short(actual)));
	    	DetailReport.recordFail(testCallerDetails, actualCallerDetails, formatErrorMsg(message, new Short(expected), new Short(actual)));
		} else {
	    	DetailReport.recordPass(testCallerDetails, actualCallerDetails, message + " expected:'" + expected + "', actual:'" + actual + "'");
		}
	}

	/**
	 * Asserts that two ints are equal. If they are not
	 * an AssertionFailedError is thrown with the given message.
	 */
  	public void assertEquals(String message, int expected, int actual) {
        StackTraceElement testCallerDetails = StackUnwinder.getCallerTestMethodDetails();
        StackTraceElement actualCallerDetails = StackUnwinder.getActualCallerDetails();

		if (expected != actual) {
			addError(formatErrorMsg(message, new Integer(expected), new Integer(actual)));
	    	DetailReport.recordFail(testCallerDetails, actualCallerDetails, formatErrorMsg(message, new Integer(expected), new Integer(actual)));
		} else {
	    	DetailReport.recordPass(testCallerDetails, actualCallerDetails, message + " expected:'" + expected + "', actual:'" + actual + "'");
		}
  	}

	/**
	 * Asserts that an object isn't null. 
	 */
	public void assertNotNull(String message, Object object) {
        StackTraceElement testCallerDetails = StackUnwinder.getCallerTestMethodDetails();
        StackTraceElement actualCallerDetails = StackUnwinder.getActualCallerDetails();

		if (object == null) {
			addError(message + " expected not null, but was null");
	    	DetailReport.recordFail(testCallerDetails, actualCallerDetails, message + " expected not null, but was null");
		} else {
	    	DetailReport.recordPass(testCallerDetails, actualCallerDetails, message + " expected not null, and was not null");
		}
	}
	
	/**
	 * Asserts that an object is null. 
	 */
	public void assertNull(String message, Object object) {
        StackTraceElement testCallerDetails = StackUnwinder.getCallerTestMethodDetails();
        StackTraceElement actualCallerDetails = StackUnwinder.getActualCallerDetails();

		if (object != null) {
			addError(message + " expected null, but was not null");
	    	DetailReport.recordFail(testCallerDetails, actualCallerDetails, message + " expected null, but was not null");
		} else {
	    	DetailReport.recordPass(testCallerDetails, actualCallerDetails, message + " expected null, and was null");
		}
	}
	
    // ------------------------------------------------------
    // ----- 
    // ------------------------------------------------------

	/**
	 * asserts list is not null and not empty
	 * @param list
	 */
	public <T extends Object> void assertListNotNullNotEmpty(String message, List<T> list) {
        StackTraceElement testCallerDetails = StackUnwinder.getCallerTestMethodDetails();
        StackTraceElement actualCallerDetails = StackUnwinder.getActualCallerDetails();
		
        if (list == null) {
			addError(message + " expected notNullNotEmpty, but was null");
	    	DetailReport.recordFail(testCallerDetails, actualCallerDetails, message + " expected notNullNotEmpty, but was null");
        } else if (list.size() == 0) {
			addError(message + " expected notNullNotEmpty, but was empty");
	    	DetailReport.recordFail(testCallerDetails, actualCallerDetails, message + " expected notNullNotEmpty, but was empty");
        } else {
	    	DetailReport.recordPass(testCallerDetails, actualCallerDetails, message + " expected notNullNotEmpty, and was notNullNotEmpty");
        }        
	}
	
	/**
	 * asserts list is not null and not empty
	 * @param list
	 */
	public void assertStrNotNullNotEmpty(String message, String str) {
        StackTraceElement testCallerDetails = StackUnwinder.getCallerTestMethodDetails();
        StackTraceElement actualCallerDetails = StackUnwinder.getActualCallerDetails();
		
        if (str == null) {
			addError(message + " expected notNullNotEmpty, but was null");
	    	DetailReport.recordFail(testCallerDetails, actualCallerDetails, message + " expected notNullNotEmpty, but was null");
        } else if (str.length() == 0) {
			addError(message + " expected notNullNotEmpty, but was empty");
	    	DetailReport.recordFail(testCallerDetails, actualCallerDetails, message + " expected notNullNotEmpty, but was empty");
        } else {
	    	DetailReport.recordPass(testCallerDetails, actualCallerDetails, message + " expected notNullNotEmpty, and was " + str);
        }        
	}
	
    // ------------------------------------------------------
    // ----- 
    // ------------------------------------------------------

	/**
	 * actual should be greater than expected
	 * 
	 * @param message
	 * @param expected
	 * @param actual
	 */
	public void assertGreaterThan(String message, long expected, long actual) {
        StackTraceElement testCallerDetails = StackUnwinder.getCallerTestMethodDetails();
        StackTraceElement actualCallerDetails = StackUnwinder.getActualCallerDetails();

       if (expected < actual) {
	    	DetailReport.recordPass(testCallerDetails, actualCallerDetails, message + " " + expected + " < " + actual);
       } else {
			addError(message + " " + expected + " < " + actual);
	    	DetailReport.recordFail(testCallerDetails, actualCallerDetails, message + " " + expected + " NOT < " + actual);
       }
	}
	/**
	 * actual should be greater than expected
	 * 
	 * @param message
	 * @param expected
	 * @param actual
	 */
	public void assertGreaterThan(String message, int expected, int actual) {
        StackTraceElement testCallerDetails = StackUnwinder.getCallerTestMethodDetails();
        StackTraceElement actualCallerDetails = StackUnwinder.getActualCallerDetails();

       if (expected < actual) {
	    	DetailReport.recordPass(testCallerDetails, actualCallerDetails, message + " " + expected + " < " + actual);
       } else {
			addError(message + " " + expected + " < " + actual);
	    	DetailReport.recordFail(testCallerDetails, actualCallerDetails, message + " " + expected + " NOT < " + actual);
       }
	}

	
	/**
	 * actual should be greater than expected
	 * 
	 * @param message
	 * @param expected
	 * @param actual
	 */
	public void assertGreaterThanEquals(String message, long expected, long actual) {
        StackTraceElement testCallerDetails = StackUnwinder.getCallerTestMethodDetails();
        StackTraceElement actualCallerDetails = StackUnwinder.getActualCallerDetails();

       if (expected <= actual) {
	    	DetailReport.recordPass(testCallerDetails, actualCallerDetails, message + " " + expected + " <= " + actual);
       } else {
			addError(message + " " + expected + " < " + actual);
	    	DetailReport.recordFail(testCallerDetails, actualCallerDetails, message + " " + expected + " NOT <= " + actual);
       }
	}
	/**
	 * actual should be greater than expected
	 * 
	 * @param message
	 * @param expected
	 * @param actual
	 */
	public void assertGreaterThanEquals(String message, int expected, int actual) {
        StackTraceElement testCallerDetails = StackUnwinder.getCallerTestMethodDetails();
        StackTraceElement actualCallerDetails = StackUnwinder.getActualCallerDetails();

       if (expected <= actual) {
	    	DetailReport.recordPass(testCallerDetails, actualCallerDetails, message + " " + expected + " <= " + actual);
       } else {
			addError(message + " " + expected + " < " + actual);
	    	DetailReport.recordFail(testCallerDetails, actualCallerDetails, message + " " + expected + " NOT <= " + actual);
       }
	}

    // ------------------------------------------------------
    // ----- 
    // ------------------------------------------------------

	/**
	 * The actual list should contain the expected object
	 * 
	 * @param expected
	 * @param actual
	 */
	public <T extends Object> void assertContains(String message, T expected, List<T> actual) {
        StackTraceElement testCallerDetails = StackUnwinder.getCallerTestMethodDetails();
        StackTraceElement actualCallerDetails = StackUnwinder.getActualCallerDetails();

        if (actual.contains(expected)) {
	    	DetailReport.recordPass(testCallerDetails, actualCallerDetails, message + " actual '" + actual.toString() + "' contains expected '" + expected + "'");
        } else {
			addError(message + " actual '" + actual.toString() + "' does NOT contain expected '" + expected + "'");
	    	DetailReport.recordFail(testCallerDetails, actualCallerDetails, message + " actual '" + actual.toString() + "' does NOT contain expected '" + expected + "'");
        }
	}
	
	/**
	 * The actual list should contain the expected object
	 * 
	 * @param expected
	 * @param actual
	 */
	public <T extends Object> void assertContains(String message, List<T> expected, T actual) {
        StackTraceElement testCallerDetails = StackUnwinder.getCallerTestMethodDetails();
        StackTraceElement actualCallerDetails = StackUnwinder.getActualCallerDetails();

        if (expected.contains(actual)) {
	    	DetailReport.recordPass(testCallerDetails, actualCallerDetails, message + " expected '" + expected.toString() + "' contains actual '" + actual + "'");
        } else {
			addError(message + " expected '" + expected.toString() + "' does NOT contain actual '" + actual + "'");
	    	DetailReport.recordFail(testCallerDetails, actualCallerDetails, message + " expected '" + expected.toString() + "' does NOT contain actual '" + actual + "'");
        }
	}

	// ------------------------------------------------------
    // ----- 
    // ------------------------------------------------------

	String formatPassMsg(String message, Object expected, Object actual) {
		String formatted= "";
		if (message != null)
			formatted= message+" ";
		return formatted+"expected:'"+expected+"' and was:'"+actual+"'";
	}
	String formatErrorMsg(String message, Object expected, Object actual) {
		String formatted= "";
		if (message != null)
			formatted= message+" ";
		return formatted+"expected:'"+expected+"' but was:'"+actual+"'";
	}


    // ------------------------------------------------------
    // ----- 
    // ------------------------------------------------------

//	private static LocalFile details = new LocalFile("details.txt");
//	static {
//		try {
//			LocalFileUtil.writeLines(details, "");
//		} catch (Exception e) {
//			
//		}
//	}
	
	
	
    // ------------------------------------------------------
    // ----- Print Format
    // ------------------------------------------------------


    /**
     * Allows you to set the print format suitable for your environment
     * 
     * @param printFormats
     */
    public static void setPrintFormats(List<PrintFormat> printFormats) {
    	if (printFormats == null || printFormats.size() == 0) {
    		sPrintFormats = null;
    		return;
    	}
    	
    	//
    	// The reason we don't juse do this assignment sPrintFormats = printFormats
    	// is a bug with using Arrays.asList(....) you can no longer add a PrintFormat 
    	// because that creates an immutable list
    	sPrintFormats = null;	// over -rides the default settings
    	for (PrintFormat pf : printFormats) {
    		addPrintFormat(pf);
    	}
	}

    /**
     * 
     * @param printFormat
     */
    public static void addPrintFormat(PrintFormat printFormat) {
    	if (sPrintFormats == null || sPrintFormats.size()==0) {
    		sPrintFormats = new ArrayList<PrintFormat>();
    	}
    	sPrintFormats.add(printFormat);
	}
    
    /**
     * 
     * @param printFormat
     */
    public static void setErrorOption(ErrorOption errorOption) {
    	sErrorOption = errorOption;
	}

    /**
     * 
     * @param printFormat
     */
    public static void setDebugOption(DebugOption debugOption) {
    	sDebugOption = debugOption;
	}

    
    /**
     * @return
     */
    private boolean isNewline() {
    	if (sPrintFormats.contains(PrintFormat.NEWLINE)) {
    		return true;
    	}
    	return false;
    }

    /**
     * @return
     */
    private boolean isBR() {
    	if (sPrintFormats.contains(PrintFormat.BR)) {
    		return true;
    	}
    	return false;
    }

    /**
     * @return
     */
    private boolean isNumberErrors() {
    	if (sPrintFormats.contains(PrintFormat.NUMBER_ERRORS)) {
    		return true;
    	}
    	return false;
    }

    /**
     * 
     */
    private void formatErrors() {
    	if (!doErrorsExist()) {
    		return;
    	}
    	String br = "";
    	String newline = "";
    	for (int i=0; i<errors.size(); i++) {
    		String str = errors.get(i);
    		int b = i + 1;
    		
    		if (isNumberErrors()) {
    			str = "(" + b + ") " + str;
    		}	//isNumberErrors

    		if (isBR()) {
    			br = "<br>";
    			str = "<br>" + str;
    		}	//isBR

    		if (isNewline()) {
    			newline = "\n";
    			str = "\n" + str;
    		}	//isNewline

    		errors.set(i, str);
    	}	//for
    	
    	//
    	//only need this extra info if there is also debug info
    	if (doDebugsExist()) {
        	errors.add(0, newline + br + "################################################");
        	errors.add(0, newline + br + "### Error details");
        	errors.add(0, newline + br + "################################################");
    	}

    }	//method

    /**
     * 
     */
    private void formatDebugs() {    	
    	if (!doDebugsExist()) {
    		return;
    	}
    	String br="";
    	String newline = "";
    	for (int i=0; i<debugs.size(); i++) {
    		String str = debugs.get(i);
    		int b = i + 1;
    		
    		if (isNumberErrors()) {
    			str = "(" + b + ") " + str;
    		}	//isNumberErrors

    		if (isBR()) {
    			str = "<br>" + str;
    			br = "<br>";
    		}	//isBR

    		if (isNewline()) {
    			str = "\n" + str;
    			newline = "\n";
    		}	//isNewline

    		debugs.set(i, str);
    	}	//for
		debugs.add(0, newline + br + "################################################");
		debugs.add(0, newline + br + "### Debug / Contextual Information");
		debugs.add(0, newline + br + "################################################");

    }	//method

    
	/**
	 * <pre>
	 * Enum class for different print formats
	 * 
	 * The order in which formatting occurs is:
	 * 
	 * 			NUMBER_ERRORS - numbers each error (1), (2) etc
	 * 			BR	- adds a "<br>" to each error so they format nicely in HTML
	 * 			NEWLINE - adds a newline "\n" char to each error so they print nicely
	 * 
	 * </pre>
	 * 
 * @author Scott Wilson - QATools.org
	 *
	 */
	public enum PrintFormat {
		NUMBER_ERRORS,
		NEWLINE,
		BR
	}

	/**
	 * <pre>
	 * Enum class for different debug options
	 * 
	 * 			QATOOLS_DETAIL_REPORT_ONLY - Debug info is ONLY displayed on the QATools Detail Report.
	 * 					This is useful if you want to disable excessive debug details from showing up on your continuous build system.
	 * 			EVERYWHERE	- This is the default. All debug info shows up in all reports, QATools, surefure, junit, etc
	 * 
	 * </pre>
	 * 
 * @author Scott Wilson - QATools.org
	 *
	 */
	public enum DebugOption {
		QATOOLS_DETAIL_REPORT_ONLY,
		EVERYWHERE
	}
	

	/**
	 * <pre>
	 * Enum class for different error options
	 * 
	 * 			SUREFIRE_SHORT - Shorten the error details in the surefire / junit reports. The formatting may be whacky in some reports.
	 * 				The QATools detail report will always include all details regardless of this setting.
	 * 				It's just a way to control how much detail shows up in your continuous build reports.
	 * 			SUREFIRE_FULL	- This is the default. Full error details show up in surefire / junit reports.
	 * 
	 * </pre>
	 * 
 * @author Scott Wilson - QATools.org
	 *
	 */
	public enum ErrorOption {
		SUREFIRE_SHORT,
		SUREFIRE_FULL
	}

	
	// TODO - equivalence "" = " " = "[]" = "()"
	// TODO - pre-process inputs, ie trim()
	
	
	
}
